#pragma once

#include "access_map.h"
#include "accessor.h"
#include "page_owner.h"

#include <mutex>

/* A file mapper, associating flexpages with file regions. */

class PageMapper : public PageOwner
{
protected:
    AccessMap _map;
    Accessor *_accessor;
    Pages *_pages;
    unsigned int _attached;
    std::mutex _lock;

    /* Internal flexpage retrieval methods. */

    Flexpage *find(offset_t offset);

    Flexpage *flexpage(offset_t offset);

public:
    explicit PageMapper(Accessor *accessor, Pages *pages);

    /* Accounting methods. */

    void attach();

    unsigned int detach();

    /* Interface for the pager. */

    Flexpage *get(offset_t offset, unsigned long flags);

    void queue(Flexpage *flexpage);

    void flush_all(offset_t start, offset_t size);

    offset_t get_data_size();

    /* Data transfer methods, implementing PageOwner. */

    void fill(Flexpage *flexpage);

    void flush(Flexpage *flexpage, bool purge);

    /* Interface for the page collection, implementing PageOwner. */

    void remove(Flexpage *flexpage);
};

// vim: tabstop=4 expandtab shiftwidth=4
